#include <stdexcept>
#include <string>

namespace fcf {
namespace except {

  struct nomem: public std::runtime_error {
    inline explicit nomem() : std::runtime_error("Not enough memory to create an object") {}
  };

  struct filelimit: public std::runtime_error {
    inline explicit filelimit() : std::runtime_error("The system limit on the total number of open files has been reached.") {}
  };

  struct incorrectipformat: public std::runtime_error {
    inline explicit incorrectipformat(const char* a_ipaddress) : std::runtime_error(std::string("Invalid IP address format '") + a_ipaddress + "'") {}
  };

  struct net_initialize: public std::runtime_error {
    template<typename Ty>
    inline explicit net_initialize(Ty a_message) : std::runtime_error(std::string("Net starup error :") + a_message) {}
  };

  struct sockettimeout: public std::runtime_error {
    inline explicit sockettimeout() : std::runtime_error("Timeout for socket operations") {}
  };

  struct connection_error: public std::runtime_error {
    inline explicit connection_error(const char* a_addr, int a_port, const char* a_message) 
      : std::runtime_error(std::string("Connection error '") + a_addr + "' on port " + std::to_string(a_port) + ":" + a_message) {}
  };

  struct socketselect: public std::runtime_error {
    inline explicit socketselect() : std::runtime_error("Select error for socket operations") {}
  };

  struct createsocket: public std::runtime_error {
    inline explicit createsocket(const char* a_message) : std::runtime_error(std::string() + "Failed to create socket. " + std::string(a_message)) {}
  };

  struct accessbind: public std::runtime_error {
    inline explicit accessbind(const char* a_addr, int a_port) : std::runtime_error(std::string("Access denied to bind socket at address ") + std::string(a_addr) + " on port "+ std::to_string(a_port)) {}
  };

  struct addrinuse: public std::runtime_error {
    inline explicit addrinuse(const char* a_addr, int a_port) : std::runtime_error(std::string("The given address is already in use ") + std::string(a_addr) + " on port "+ std::to_string(a_port)) {}
  };

  struct errorbind: public std::runtime_error {
    inline explicit errorbind(const char* a_addr, int a_port, const char* a_message) : std::runtime_error(std::string("Failed to bind to address ") + std::string(a_addr) + " on port "+ std::to_string(a_port) + ". " + std::string(a_message)) {}
  };

  struct listenerror: public std::runtime_error {
    inline explicit listenerror(const char* a_addr, int a_port, const char* a_message) : std::runtime_error(std::string("Failed to listen to address ") + std::string(a_addr) + " on port " + std::to_string(a_port) + ". " + std::string(a_message)) {}
  };

  struct socketagain: public std::runtime_error {
    inline explicit socketagain() : std::runtime_error("The socket is marked non-blocking and the requested operation would block or a receive timeout had been set and the timeout expired before data was received") {}
  };

  struct connreset: public std::runtime_error {
    inline explicit connreset() : std::runtime_error("Connection reset by peer.") {}
  };

  struct socketsend: public std::runtime_error {
    inline explicit socketsend(const char* a_msg) : std::runtime_error(std::string() + "Data sending error. " + std::string(a_msg)) {}
  };

  struct socketempty: public std::runtime_error {
    inline explicit socketempty() : std::runtime_error(std::string() + "Empty data on socket.") {}
  };

  struct connrefused: public std::runtime_error {
    inline explicit connrefused() : std::runtime_error("A remote host refused to allow the network connection.") {}
  };

  struct socketrec: public std::runtime_error {
    inline explicit socketrec(const char* a_msg) : std::runtime_error(std::string() + "Error getting data from socket. " + std::string(a_msg)) {}
  };

  struct nouserinfo: public std::runtime_error {
    inline explicit nouserinfo(const char* a_user_name) : std::runtime_error(std::string("Failed to get information about user '") + a_user_name + "'") {}
  };

  struct authuser : public std::runtime_error {
    inline explicit authuser(const char* a_user_name) : std::runtime_error(std::string("Failed to authenticate for user '") + a_user_name + "'") {}
  };

  struct process_timeout: public std::runtime_error {
    inline explicit process_timeout(const char* a_cmdline) : std::runtime_error(std::string() + "The process startup timeout has expired '" + std::string(a_cmdline) + "'") {}
  };

  struct process_start: public std::runtime_error {
    inline explicit process_start(const char* a_cmdline) : std::runtime_error(std::string() + "Failed to start the process: '" + std::string(a_cmdline) + "'") {}
  };

  struct max_request_timeout: public std::runtime_error {
    inline explicit max_request_timeout(const char* a_address, int a_port) 
      : std::runtime_error(std::string() + "(IP:" + a_address + "; PORT: " + std::to_string(a_port) + ") The timeout of the request.") {}
  };

  struct max_request_timeout_fi: public std::runtime_error {
    inline explicit max_request_timeout_fi(const char* a_address, int a_port, const char* a_host, const char* a_path) 
      : std::runtime_error(std::string() + "(IP:" + a_address + "; PORT: " + std::to_string(a_port) + "; HOST: " + a_host+ "; PATH: " + a_path + ") The timeout of the request.") {}
  };

  struct max_response_timeout_fi: public std::runtime_error {
    inline explicit max_response_timeout_fi(const char* a_address, int a_port, const char* a_host, const char* a_path) 
      : std::runtime_error(std::string() + "(IP:" + a_address + "; PORT: " + std::to_string(a_port) + "; HOST: " + a_host+ "; PATH: " + a_path + ") The timeout of the response.") {}
  };

}
}
